﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;

namespace Sokoban
{
    public partial class Form1 : Form
    {
        private Visualizer vis;
        private Board b;
        private List<Direction> solution;
        private HeuristicSolver<SokobanState, SokobanProblem> solver;
        public static bool useDeadEndDetection = true,
            useAdvancedSuccessorsGeneration = true,
			useSymmetryBreaking = true;
        private Heuristic<SokobanState, SokobanProblem> heuristic;

        public Form1()
        {
            //  ZDE NASTAVTE SVOU HEURISTIKU

            //heuristic = new BlindHeuristic<SokobanState, SokobanProblem>();
            heuristic = new SimpleDistanceHeuristic();
            //heuristic = new AdvancedDistanceHeuristic();
            //heuristic = new WeightedHeuristic<SokobanState, SokobanProblem>(new AdvancedDistanceHeuristic(), 10);

            


            // zbytek neni treba editovat

            solver = new Astar<SokobanState, SokobanProblem>();
            //solver = new GreedyBFS<SokobanState, SokobanProblem>();

            solver.SetHeuristic(heuristic);
            InitializeComponent();
            vis = new Visualizer(pictureBox1, label1);
            b = new Board();
            b.size = new Coords(15, 4);
            useSymmetryBreaking = checkBox3.Checked;
            useDeadEndDetection = checkBox1.Checked;
            useAdvancedSuccessorsGeneration = checkBox2.Checked;

        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (int.Parse(SeedtextBox.Text) < 0)
                b.CreateSimple();
            else
                b.Randomize(int.Parse(SeedtextBox.Text));
            vis.Visualize(b);
        }

        private void leftButton_Click(object sender, EventArgs e)
        {
            b.Move(Direction.left);
            vis.Visualize(b);
        }

        private void upBotton_Click(object sender, EventArgs e)
        {
            b.Move(Direction.up);
            vis.Visualize(b);
        }

        private void downButton_Click(object sender, EventArgs e)
        {
            b.Move(Direction.down);
            vis.Visualize(b);
        }

        private void rightButton_Click(object sender, EventArgs e)
        {
            b.Move(Direction.right);
            vis.Visualize(b);
        }

        private void Solvebutton_Click(object sender, EventArgs e)
        {
            solvingLabel.Text = "SOLVING ... ";

            SokobanProblem p = new SokobanProblem(useSymmetryBreaking: Form1.useSymmetryBreaking, useAdvancedSolving: Form1.useAdvancedSuccessorsGeneration,
                useDeadEndDetection: Form1.useDeadEndDetection, board: b);
            solution = p.ExtractSolution(solver.Solve(p, TimeSpan.FromSeconds(300)));
            if (solution == null)
                solvingLabel.Text = "solution NOT found";
            else solvingLabel.Text = "solution found";
        }

        private void ShowSolutiontimer_Tick(object sender, EventArgs e)
        {
            if (solution == null || solution.Count == 0)
            {
                ShowSolutiontimer.Enabled = false;
                return;
            }
            b.Move(solution[0]);
            solution.RemoveAt(0);
            vis.Visualize(b);
        }

        private void ShowSolutionbutton_Click(object sender, EventArgs e)
        {
            ShowSolutiontimer.Enabled = true;
			if (solution == null || solution.Count == 0)
				return;
			int i = 1;
			foreach (var item in solution)
			{
				Console.WriteLine(i++ + "\t" + item.ToString());
			}
        }

        private void Save_button_Click(object sender, EventArgs e)
        {
            saveMap_saveFileDialog.ShowDialog();
        }

        private void saveMap_saveFileDialog_FileOk(object sender, CancelEventArgs e)
        {
            b.Save(saveMap_saveFileDialog.FileName);
        }

        private void Load_button_Click(object sender, EventArgs e)
        {
            LoadMap_openFileDialog.ShowDialog();
        }

        private void LoadMap_openFileDialog_FileOk(object sender, CancelEventArgs e)
        {
            b.Load(LoadMap_openFileDialog.FileName);
            vis.Visualize(b);
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
			useDeadEndDetection = checkBox1.Checked;
			if (vis != null && b != null)
				vis.Visualize(b);
        }

		private void checkBox3_CheckedChanged(object sender, EventArgs e)
		{
			useSymmetryBreaking = checkBox3.Checked;
		}

		private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {
			useAdvancedSuccessorsGeneration = checkBox2.Checked;
        }
    }
}
